#include "tester.h"
#include "ui_tester.h"
#include <QDate>

Tester::Tester(QWidget *parent)
    : QMainWindow(parent)
    , ui(new Ui::Tester)
{
    ui->setupUi(this);
    ui->textBrowser->setFontPointSize(14);

    QPalette qpalette;
    qpalette.setColor(this->backgroundRole(), QColor("#FFCCFF"));
    this->setAutoFillBackground(true);
    this->setPalette(qpalette);

    QFont font = ui->label->font();
    font.setPointSize(10);
    ui->label->setFont(font);

    loginWidget = new LoginWidget();
    testSelectionWidget = new TestSelection();

    connect(loginWidget, &LoginWidget::onClosed, this, &Tester::receiveNameAndPassword);
    connect(loginWidget, &LoginWidget::onClosed, testSelectionWidget, &TestSelection::_sh);
    connect(testSelectionWidget, &TestSelection::onClosed, this, &Tester::receiveTestEntity);
    connect(this->ui->pushButton, &QPushButton::clicked, this, &Tester::startExecution);
    connect(testSelectionWidget, &TestSelection::onClosed, this, &Tester::startExecution);

    this->show();
    loginWidget->show();
}

Tester::~Tester()
{
    delete ui;
}

void Tester::receiveNameAndPassword()
{
    this->currentName = loginWidget->getName();
    this->currentPassword = loginWidget->getPassword();
}

void Tester::receiveTestEntity()
{
    this->currentTest = this->testSelectionWidget->getTestEntity();

    questCount = currentTest->getQuestions().size();

    emit(onDataReceived());
}

void Tester::startExecution()
{
    connect(ui->pushButton, &QPushButton::clicked, this, &Tester::getAnswerFromLineEdit);
    connect(ui->pushButton, &QPushButton::clicked, this->ui->lineEdit, &QLineEdit::clear);
    if (counter < questCount) {
        QVector<QuestionEntity> questVector = currentTest->getQuestions();
        QuestionEntity currentQuestion = questVector[counter];
        drawCurrentQuestion(currentQuestion);
    }
    else {
        getAnswerFromLineEdit();

        this->currentTest->checkQuestions(this->currentAnswers);
        int percentage = currentTest->getPercentage();
        ui->lineEdit->hide();
        ui->label->hide();
        ui->textBrowser->setText("Тест закончен, спасибо за участие! \nВаш результат: " + QString::number(percentage));

        QString date = QString::number(QDate::currentDate().year()) + "-"
                + QString::number(QDate::currentDate().month())
                + "-" + QString::number(QDate::currentDate().day());

        qDebug() << date;

        QSqlQuery query(db);
        query.exec("INSERT INTO test_results VALUES ('" + currentName + "',"
                   + QString::number(percentage)
                   + "," + "'" + date + "'," + QString::number(currentTest->getId()) + ");");
        qDebug() << "INSERT INTO test_results VALUES ('" + currentName + "',"
                    + QString::number(percentage)
                    + "," + "'" + date + "'," + QString::number(currentTest->getId()) + ");";

        ui->pushButton->setText("Закончить");
        disconnect(this->ui->pushButton, &QPushButton::clicked, this, &Tester::startExecution);
        connect(ui->pushButton, &QPushButton::clicked, this, &Tester::close);
    }
}

void Tester::getAnswerFromLineEdit()
{
    QString textAnswer = ui->lineEdit->text();
    QStringList answerList = ui->lineEdit->text().split(" ");
    QSet<QString> answers(answerList.begin(), answerList.end());

    currentAnswers.push_back(answers);
}

void Tester::drawCurrentQuestion(const QuestionEntity &currentQuestion)
{
    QString testString;

    testString += currentQuestion.getQuestion() + "\n";
    for(auto& var: currentQuestion.getVariants()) {
        testString += "  " + var + "\n";
    }

    this->ui->textBrowser->setText(testString);

    counter++;
}
